
/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file    app_filex.c
  * @author  MCD Application Team
  * @brief   FileX applicative file
  ******************************************************************************
  * @attention
  *
  * Copyright (c) 2020-2021 STMicroelectronics.
  * All rights reserved.
  *
  * This software is licensed under terms that can be found in the LICENSE file
  * in the root directory of this software component.
  * If no LICENSE file comes with this software, it is provided AS-IS.
  *
  ******************************************************************************
  */
/* USER CODE END Header */

/* Includes ------------------------------------------------------------------*/
#include "app_filex.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include "Debug.h"
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* Main thread stack size */
#define FX_APP_THREAD_STACK_SIZE         1024
/* Main thread priority */
#define FX_APP_THREAD_PRIO               10

/* USER CODE BEGIN PD */

/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/
/* Main thread global data structures.  */
TX_THREAD       fx_app_thread;

/* USER CODE BEGIN PV */
#define CUSTOM_DRIVER_ID           0xDDDD
#define NAND_CUSTOM_DRIVER_NAME    "NAND CUSTOM DRIVER"
#include "lx_stm32_nand_custom_driver.h"

#define LX_NAND_CUSTOM_DRIVERS   {.name = NAND_CUSTOM_DRIVER_NAME,  .id = CUSTOM_DRIVER_ID, .nand_driver_initialize = lx_stm32_nand_custom_driver_initialize}

#define MAX_LX_NAND_DRIVERS     1
#define UNKNOWN_DRIVER_ID        0xFFFFFFFF

#define FX_NAND_DISK_HIDDEN_SECTORS                0

/* Heads */
#define FX_NAND_DISK_HEADS                         1

/* Sectors per track */
#define FX_NAND_DISK_SECTORS_PER_TRACK             1

/* Length of the message to be written in the file */
#define FX_NAND_DISK_LENGTH                        28

/* Total sectors */
#define FX_NAND_DISK_TOTAL_SECTORS                (((TOTAL_BLOCKS-1) * PHYSICAL_PAGES_PER_BLOCK * BYTES_PER_PHYSICAL_PAGE)/ FX_NAND_DISK_SECTOR_SIZE)

/* Number of FATs */
#define FX_NAND_DISK_NUMBER_OF_FATS                1

/* Directory Entries */
#define FX_NAND_DISK_DIRECTORY_ENTRIES             32

/* Sector size */
#define FX_NAND_DISK_SECTOR_SIZE                   BYTES_PER_PHYSICAL_PAGE /* sector size is always the page size of the underlying NAND hardware */

/* Sectors per cluster */
#define FX_NAND_DISK_SECTORS_PER_CLUSTER           1
struct fx_lx_nand_driver_instance
{
    LX_NAND_FLASH flash_instance;

    CHAR name[32];

    UINT id;

    UINT (*nand_driver_initialize)(LX_NAND_FLASH *);

    UINT initialized;

};

static struct fx_lx_nand_driver_instance  fx_lx_nand_drivers[MAX_LX_NAND_DRIVERS] =
{
    LX_NAND_CUSTOM_DRIVERS

};
///* Memory buffer size should be at least 7 * total block count + 2 * page size,
//   that is 7 * 1024 + 2 * 528 = 8224 bytes *///2112*2= 4224--11392
__ALIGN_BEGIN ULONG               lx_memory_buffer[(7808 * 2) / sizeof (ULONG)]__ALIGN_END;

static struct fx_lx_nand_driver_instance *current_driver = NULL;

VOID  fx_mystm32_levelx_nand_driver(FX_MEDIA *media_ptr)
{
    ULONG i;
    UINT status;
    UCHAR *source_buffer;
    UCHAR *destination_buffer;
    ULONG logical_sector;

    /* Process the driver request specified in the media control block.*/

    current_driver = &fx_lx_nand_drivers[0];

    switch(media_ptr->fx_media_driver_request)
    {

        case FX_DRIVER_INIT:
            {
                if (current_driver->initialized == FX_FALSE)
                {
                    /* Open flash instance*/
                    status = lx_nand_flash_open(&current_driver->flash_instance, current_driver->name, current_driver->nand_driver_initialize,lx_memory_buffer, sizeof(lx_memory_buffer));

                    /* LevelX driver correctly initialized */
                    if (status == LX_SUCCESS)
                    {
                        current_driver->initialized = FX_TRUE;
                        media_ptr->fx_media_driver_status = FX_SUCCESS;

                        media_ptr->fx_media_driver_free_sector_update = FX_TRUE;
                        break;
                    }
                    else
                    {
                        media_ptr->fx_media_driver_status = FX_IO_ERROR;
                        ERROR_MSG("lx_nand_flash_open fail ---%X\r\n",status);
#if 1
							status =  lx_nand_flash_format(&current_driver->flash_instance, current_driver->name, current_driver->nand_driver_initialize,lx_memory_buffer, sizeof(lx_memory_buffer));
							if (status != LX_SUCCESS)
							{

								/* Return an I/O error to FileX.  */
								ERROR_MSG("forma nand flash Fial\r\n");

								return;
							}
							current_driver->initialized = FX_TRUE;
							media_ptr->fx_media_driver_status = FX_SUCCESS;

							media_ptr->fx_media_driver_free_sector_update = FX_TRUE;
#endif
							break;
                    }
                }
                else
                {
                    media_ptr->fx_media_driver_status = FX_SUCCESS;
                }

                break;
            }

        case FX_DRIVER_UNINIT:
            {
                /* Successful driver */
                status = lx_nand_flash_close(&current_driver->flash_instance);

                if (status == LX_SUCCESS)
                {
                    media_ptr->fx_media_driver_status =  FX_SUCCESS;
                }
                else
                {
                    media_ptr->fx_media_driver_status =  FX_IO_ERROR;
                }

                break;
            }

        case FX_DRIVER_READ:
            {
                /* Setup the destination buffer and logical sector.  */
                logical_sector = media_ptr->fx_media_driver_logical_sector;
                destination_buffer =(UCHAR *)media_ptr->fx_media_driver_buffer;

                /* Loop to read sectors from flash.  */
                for (i = 0; i < media_ptr->fx_media_driver_sectors; i++)
                {

                    /* Read a sector from NAND flash.  */
                    status =  lx_nand_flash_sector_read(&current_driver->flash_instance, logical_sector, destination_buffer);

                    /* Determine if the read was successful.  */
                    if (status != LX_SUCCESS)
                    {

                        /* Return an I/O error to FileX.  */
                        media_ptr->fx_media_driver_status = FX_IO_ERROR;

                        return;
                    }

                    /* Move to the next entries.  */
                    logical_sector++;
                    destination_buffer = destination_buffer + media_ptr->fx_media_bytes_per_sector;
                }

                /* Successful driver request.  */
                media_ptr->fx_media_driver_status =  FX_SUCCESS;

                break;
            }

        case FX_DRIVER_BOOT_READ:
            {

                /* Read the boot record and return to the caller.  */

                /* Setup the destination buffer.  */
                destination_buffer =  (UCHAR *) media_ptr -> fx_media_driver_buffer;

                /* Read boot sector from NAND flash.  */
                status =  lx_nand_flash_sector_read(&current_driver->flash_instance, 0, destination_buffer);

                /* Determine if the boot read was successful.  */
                if (status != LX_SUCCESS)
                {
                    /* Return an I/O error to FileX.  */
                    media_ptr -> fx_media_driver_status =  FX_IO_ERROR;

                    return;
                }

                /* Successful driver request.  */
                media_ptr -> fx_media_driver_status =  FX_SUCCESS;
                break;
            }

        case FX_DRIVER_WRITE:
            {
                /* Setup the source buffer and logical sector.  */
                logical_sector = media_ptr->fx_media_driver_logical_sector;
                source_buffer = (UCHAR *) media_ptr->fx_media_driver_buffer;

                /* Loop to write sectors to flash.  */
                for (i = 0; i < media_ptr->fx_media_driver_sectors; i++)
                {
                    /* Write a sector to NAND flash.  */
                    status =  lx_nand_flash_sector_write(&current_driver->flash_instance, logical_sector, source_buffer);

                    /* Determine if the write was successful.  */
                    if (status != LX_SUCCESS)
                    {
                        /* Return an I/O error to FileX.  */
                        media_ptr->fx_media_driver_status =  FX_IO_ERROR;
                        return;
                    }

                    /* Move to the next entries.  */
                    logical_sector++;
                    source_buffer =  source_buffer + media_ptr->fx_media_bytes_per_sector;
                }

                /* Successful driver request.  */
                media_ptr->fx_media_driver_status =  FX_SUCCESS;
                break;
            }

        case FX_DRIVER_BOOT_WRITE:
            {

                /* Write the boot record and return to the caller.  */

                /* Setup the source buffer.  */
                source_buffer =       (UCHAR *) media_ptr -> fx_media_driver_buffer;

                /* Write boot sector to NAND flash.  */
                status =  lx_nand_flash_sector_write(&current_driver->flash_instance, 0, source_buffer);

                /* Determine if the boot write was successful.  */
                if (status != LX_SUCCESS)
                {

                    /* Return an I/O error to FileX.  */
                    media_ptr -> fx_media_driver_status =  FX_IO_ERROR;

                    return;
                }

                /* Successful driver request.  */
                media_ptr -> fx_media_driver_status =  FX_SUCCESS;
                break ;
            }
        case FX_DRIVER_RELEASE_SECTORS:
            {
                /* Setup the logical sector.  */
                logical_sector =  media_ptr->fx_media_driver_logical_sector;

                /* Release sectors.  */
                for (i = 0; i < media_ptr->fx_media_driver_sectors; i++)
                {
                    /* Release NAND flash sector.  */
                    status = lx_nand_flash_sector_release(&current_driver->flash_instance, logical_sector);

                    /* Determine if the sector release was successful.  */
                    if (status != LX_SUCCESS)
                    {
                        /* Return an I/O error to FileX.  */
                        media_ptr->fx_media_driver_status = FX_IO_ERROR;
                        return;
                    }

                    /* Move to the next entries.  */
                    logical_sector++;
                }

                /* Successful driver request.  */
                media_ptr->fx_media_driver_status =  FX_SUCCESS;
                break;
            }

        case FX_DRIVER_FLUSH:
            {
                /* Return driver success.  */
                media_ptr->fx_media_driver_status =  FX_SUCCESS;
                break;
            }

        case FX_DRIVER_ABORT:
            {
                /* Return driver success.  */
                media_ptr->fx_media_driver_status =  FX_SUCCESS;
                break;
            }

        default:
            {
                /* Invalid driver request.  */
                media_ptr->fx_media_driver_status =  FX_IO_ERROR;
                break;
            }
    }
}

UCHAR media_memory[2048];

/* Define FileX global data structures.  */
FX_MEDIA        nand_flash_disk;
FX_FILE         fx_file;
UCHAR testbuf[2048];
/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
/* Main thread entry function.  */
void fx_app_thread_entry(ULONG thread_input);

/* USER CODE BEGIN PFP */

/* USER CODE END PFP */

/**
  * @brief  Application FileX Initialization.
  * @param memory_ptr: memory pointer
  * @retval int
  */
UINT MX_FileX_Init(VOID *memory_ptr)
{
  UINT ret = FX_SUCCESS;

  TX_BYTE_POOL *byte_pool = (TX_BYTE_POOL*)memory_ptr;
  VOID *pointer;

  /* USER CODE BEGIN MX_FileX_MEM_POOL */

  /* USER CODE END MX_FileX_MEM_POOL */

  /* USER CODE BEGIN 0 */

  /* USER CODE END 0 */

  /*Allocate memory for the main thread's stack*/
  ret = tx_byte_allocate(byte_pool, &pointer, FX_APP_THREAD_STACK_SIZE, TX_NO_WAIT);

  /* Check FX_APP_THREAD_STACK_SIZE allocation*/
  if (ret != FX_SUCCESS)
  {
    return TX_POOL_ERROR;
  }

  /* Create the main thread.  */
  ret = tx_thread_create(&fx_app_thread, FX_APP_THREAD_NAME, fx_app_thread_entry, 0, pointer, FX_APP_THREAD_STACK_SIZE,
                         FX_APP_THREAD_PRIO, FX_APP_PREEMPTION_THRESHOLD, FX_APP_THREAD_TIME_SLICE, FX_APP_THREAD_AUTO_START);

  /* Check main thread creation */
  if (ret != FX_SUCCESS)
  {
    return TX_THREAD_ERROR;
  }
  /* USER CODE BEGIN MX_FileX_Init */

  /* USER CODE END MX_FileX_Init */

  /* Initialize FileX.  */
  fx_system_initialize();

  /* USER CODE BEGIN MX_FileX_Init 1*/

  /* USER CODE END MX_FileX_Init 1*/

  return ret;
}

 /**
 * @brief  Main thread entry.
 * @param thread_input: ULONG user argument used by the thread entry
 * @retval none
 */
void fx_app_thread_entry(ULONG thread_input)
{
  /* USER CODE BEGIN fx_app_thread_entry 0 */
	  UINT status;
	  ULONG bytes_read;
	  ULONG available_space_pre;
	  ULONG available_space_post;
	  CHAR read_buffer[33];
	  CHAR data[] = "This is FileX working on HDSMART";
  /* USER CODE END fx_app_thread_entry 0 */
	  return;
  /* USER CODE BEGIN fx_app_thread_entry 1 */
#if 0
    status =  fx_media_format(&nand_flash_disk,
			  fx_mystm32_levelx_nand_driver,         /* Driver entry */
	                              (VOID *)CUSTOM_DRIVER_ID,          /* NAND disk memory pointer */
	                              media_memory,                        /* Media buffer pointer */
	                              sizeof(media_memory),                /* Media buffer size */
	                              "NAND_FLASH_DISK",                   /* Volume Name */
	                              FX_NAND_DISK_NUMBER_OF_FATS,         /* Number of FATs */
	                              FX_NAND_DISK_DIRECTORY_ENTRIES,      /* Directory Entries */
	                              FX_NAND_DISK_HIDDEN_SECTORS,         /* Hidden sectors */
	                              FX_NAND_DISK_TOTAL_SECTORS,          /* Total sectors */
	                              FX_NAND_DISK_SECTOR_SIZE,            /* Sector size */
	                              FX_NAND_DISK_SECTORS_PER_CLUSTER,    /* Sectors per cluster */
	                              FX_NAND_DISK_HEADS,                  /* Heads */
	                              FX_NAND_DISK_SECTORS_PER_TRACK);     /* Sectors per track */

    /* Check if the format status */
    if (status != FX_SUCCESS)
    {
        ERROR_MSG("FileX Test Fail---%X\r\n",status);
        return;
    }
#endif
    status =  fx_media_open(&nand_flash_disk, "FX_LX_NAND_DISK", fx_mystm32_levelx_nand_driver, (VOID *)CUSTOM_DRIVER_ID, media_memory, sizeof(media_memory));

      /* Check the media open status.  */
      if (status != FX_SUCCESS)
      {
          ERROR_MSG("FileX media open Fail === 0x%X\r\n",status);
          return;
      }
#if 0
      status =  fx_media_space_available(&nand_flash_disk, &available_space_pre);
      DEBUG_MSG("User available NAND Flash disk space size before file is written: %lu bytes.\r\n", available_space_pre);

     status = fx_file_create(&nand_flash_disk, "HDsmart.txt");
       INFO_MSG("create = 0x%X\r\n",status);
      if ((status == FX_ALREADY_CREATED) ||  (status == FX_SUCCESS))
      {
           status =  fx_file_open(&nand_flash_disk, &fx_file, "HDsmart.txt", FX_OPEN_FOR_WRITE);

        /* Check the file open status.  */
        if (status != FX_SUCCESS)
        {
          /* Error opening file, call error handler.  */
            ERROR_MSG("FileX Test Fail\r\n");
            return;
        }
        status =  fx_file_seek(&fx_file, 0);
        if (status != FX_SUCCESS)
        {
          /* Error performing file seek, call error handler.  */
            ERROR_MSG("FileX Test Fail\r\n");
            return;
        }
        status = fx_file_write(&fx_file, data, sizeof(data));
        if (status != FX_SUCCESS)
        {
          /* Error reading file, call error handler.  */
            ERROR_MSG("FileX Test Fail\r\n");
            return;
        }
        status =  fx_file_close(&fx_file);
        if (status != FX_SUCCESS)
        {
          /* Error closing the media, call error handler.  */
            ERROR_MSG("FileX Test ALREADY CREATED\r\n");
            return;
        }
      }

#else
	        status =  fx_file_open(&nand_flash_disk, &fx_file, "HDsmart.txt", FX_OPEN_FOR_READ);

	        /* Check the file open status.  */
	        if (status != FX_SUCCESS)
	        {
	          /* Error opening file, call error handler.  */
	        	ERROR_MSG("FileX Open File fail == 0x%X\r\n",status);
	        	return;
	        }

	        /* Seek to the beginning of the test file.  */
	        status =  fx_file_seek(&fx_file, 0);

	        /* Check the file seek status.  */
	        if (status != FX_SUCCESS)
	        {
	          /* Error performing file seek, call error handler.  */
	        	ERROR_MSG("FileX Test Fail\r\n");
	        	return;
	        }

	        /* Read the first 28 bytes of the test file.  */
	        status =  fx_file_read(&fx_file, testbuf, 32, &bytes_read);
	        DEBUG_MSG("FileX Read bytes_read: %d\r\n", bytes_read);
	        /* Check the file read status.  */
	        if ((status != FX_SUCCESS) || (bytes_read != 32))
	        {
	          /* Error reading file, call error handler.  */
	        	ERROR_MSG("FileX Test Fail\r\n");
	        	return;
	        }
        testbuf[32] = 0;
        DEBUG_MSG("FileX Read: %s\r\n", testbuf);
	        /* Close the test file.  */
	        status =  fx_file_close(&fx_file);
#endif

      status = fx_directory_first_entry_find(&nand_flash_disk,(char *)testbuf);
        while(status == FX_SUCCESS)
        {
            INFO_MSG("%s\r\n",testbuf);
            status = fx_directory_next_entry_find(&nand_flash_disk,(char *)testbuf);
        }
        status =  fx_media_space_available(&nand_flash_disk, &available_space_post);
    status =  fx_media_close(&nand_flash_disk);

    /* Check the media close status.  */
    if (status != FX_SUCCESS)
    {
      /* Error closing the media, call error handler.  */
        ERROR_MSG("FileX Test Fail\r\n");
        return;
    }
    while (1)
    {
        INFO_MSG("User's remaining NAND Flash disk space: %lu bytes.\r\n", available_space_post);
        tx_thread_sleep(5000);
    }
  /* USER CODE END fx_app_thread_entry 1 */
}

/* USER CODE BEGIN 1 */

/* USER CODE END 1 */
