#include "buzzer.h"

#if 1

/** @brief 蜂鸣器参数结构 */
typedef	struct		
{
	UINT8			u8BuzzData;	///<传输音乐数据(电源板发音方案禁止输出0)
	UINT8			u8BuzzerID;	///<蜂鸣器ID
	UINT16			u8Buzzfreq;	///<频率
	UINT8			u8Duration;	///<时长
	BuzzType_E		enBuzzType;	///<蜂鸣类型（BUZZ_SINGLE/BUZZ_MUSIC）
	BuzzStates_E	enStates;	///<开停状态（BUZZ_DIS/BUZZ_ON/BUZZ_OFF）
	MusicPara_T	    *Melody;	///<指向曲谱对应音符
}BuzzPara_T;	

static BuzzPara_T stcRunBuff;	///<蜂鸣器参数（模块内部数据）
void BuzzInit(void);
void Buzzer_Init(void)
{
	/*IO初始化*/
    BuzzInit();
	INFO_MSG("Buzzer Init Ok\r\n");
}
/*
***************************************************************************

// 函数名称: void BuzzParaInit(void)
// 输入参数: 无
// 输出参数: 无
// 返回数值: 无 
// 函数功能: 蜂鸣参数初始化函数
***************************************************************************
*/
void BuzzParaInit(void)
{
	stcRunBuff.u8BuzzData = 0u;				//非电源板发音方案，初始化传输蜂鸣数据为0
	stcRunBuff.u8BuzzerID = 0u;				//初始默认蜂鸣器ID=0
	stcRunBuff.enBuzzType = BUZZ_SINGLE;	//初始默认蜂鸣类型为“单音”
	stcRunBuff.enStates = BUZZ_DIS;			//初始默认状态为“静音”
	stcRunBuff.u8Buzzfreq = EOT;			//初始默认音频为“休止符”
	stcRunBuff.u8Duration = 0u;				//初始默认时长为0
	stcRunBuff.Melody = NULL;	//初始默认曲目表首地址
}
//================================================================================================================
// 函数名称: void BuzzInit(void)
// 输入参数: 无
// 输出参数: 无
// 返回数值: 无 
// 函数功能: 蜂鸣器初始化函数（main.c调用）
//================================================================================================================

void BuzzInit(void)
{
    HAL_GPIO_WritePin(GPIOE, GPIO_PIN_5, GPIO_PIN_RESET);
    HAL_TIM_PWM_Start(&htim15, TIM_CHANNEL_2);

	BuzzParaInit();		//蜂鸣参数初始化
}

//================================================================================================================
// 函数名称: void BuzzOnOver(void)
// 输入参数: 无
// 输出参数: 无
// 返回数值: 无   
// 函数功能: 曲谱播放时长结束处理
//================================================================================================================
void BuzzOnOver(void)
{
	stcRunBuff.enStates = BUZZ_OFF;	//状态设置为“停顿”
	stcRunBuff.u8Duration = (*(stcRunBuff.Melody)).u8OffTime;	//音长设置为“停顿时间”	
}
//================================================================================================================
// 函数名称: void BuzzOffOver(void)
// 输入参数: 无
// 输出参数: 无
// 返回数值: 无   
// 函数功能: 曲谱停顿时长结束处理
//================================================================================================================
void BuzzOffOver(void)
{
	(stcRunBuff.Melody)++;
	if( EOT == stcRunBuff.u8Buzzfreq )
	{
		BuzzInit();	//初始化蜂鸣器
		INFO_MSG("Buzz Off Over\r\n");
	}
	else
	{
		stcRunBuff.u8Buzzfreq = (*(stcRunBuff.Melody)).u8Pitch;		//音频设为下一个“音符”
		stcRunBuff.u8Duration = (*(stcRunBuff.Melody)).u8OnTime;	//音长设置为“播放时长”
		stcRunBuff.enStates = BUZZ_ON;								//状态设置为“播放”
		BUZZ_FRQ_SET(stcRunBuff.u8Buzzfreq);							//蜂鸣频率设置
	}
}
//================================================================================================================
// 函数名称: BOOLEAN BuzzMusicSet(en_MusicType enType)
// 输入参数: en_MusicType enType曲谱类型（枚举型变量）
// 输出参数: 无
// 返回数值: 执行成功与否（布尔型）   
// 函数功能: 蜂鸣器曲谱设置函数：按照输入的音乐类型找到对应曲谱
//================================================================================================================
UINT8 BuzzMusicSet(MusicPara_T *_musicData)
{
#if( SOUND_CHORD == SOUND_TYPE || SOUND_SIGLE == SOUND_TYPE )
	if( _musicData == NULL)
	{
		stcRunBuff.enStates = BUZZ_DIS;	
		return	NOK; 				//找不到曲谱并返回操作失败
	}
	
	stcRunBuff.Melody = _musicData;			//按照音乐类型找到对应曲谱
	
	if( (0u == (*(stcRunBuff.Melody)).u8Pitch )|| (EOT <= (*(stcRunBuff.Melody)).u8Pitch))
	{
		stcRunBuff.Melody = NULL;	//初始默认曲目表首地址
		stcRunBuff.enStates = BUZZ_DIS;	
        ERROR_MSG("u8Pitch : 0x%X\r\n",(*(stcRunBuff.Melody)).u8Pitch );
		return	NOK; 				//音频超限，返回操作失败	
	}
	else
	{
		stcRunBuff.enBuzzType = BUZZ_MUSIC;				//设置蜂鸣类型为“音乐”
		stcRunBuff.enStates = BUZZ_ON;					//设置状态为“播放”
	
		stcRunBuff.u8Buzzfreq = (*(stcRunBuff.Melody)).u8Pitch;		//将曲谱首音音频、时长赋值给蜂鸣器参数
		stcRunBuff.u8Duration = (*(stcRunBuff.Melody)).u8OnTime;

		BUZZ_FRQ_SET(stcRunBuff.u8Buzzfreq);				//蜂鸣频率设置
		//INFO_MSG("u8Buzzfreq : %d\r\n",stcRunBuff.u8Buzzfreq );
		return	OK; 	
	}
#else
	if( 0x0Fu < enType )					//不能超过15个音乐
	{
		stcRunBuff.u8RemainTime = 0u;	//设置蜂鸣保持时间为0秒
		stcRunBuff.u8BuzzData = 0u;		//传输蜂鸣数据为0
		stcRunBuff.enStates = BUZZ_DIS;
		return	FALSE; 					//找不到曲谱并返回操作失败
	}
	else
	{
		stcRunBuff.u8RemainTime = 100u;	//设置蜂鸣保持时间为1秒
		stcRunBuff.u8BuzzData = (INT8U)(++stcRunBuff.u8DifCnt<<4u)|enType;	//生成传输蜂鸣数据	
		stcRunBuff.enStates = BUZZ_ON;					//设置状态为“播放”
		return	OK; 				//找不到曲谱并返回操作失败
	}
#endif
}

/*
***************************************************************************
*	功  能：蜂鸣器
*	输	入：无
*	输	出：无
*	返回值：无
***************************************************************************
*/
void Buzzer_Process()
{
    int pos = 0,value;
    while(1)
    {
#if( SOUND_CHORD == SOUND_TYPE || SOUND_SIGLE == SOUND_TYPE )
        if( BUZZ_DIS == stcRunBuff.enStates )   //音频超限（单音）或找不到对应曲谱（音乐）：关闭蜂鸣器并退出
        {
            BuzzInit(); //初始化蜂鸣器    
            return; 
        }
        switch( stcRunBuff.enBuzzType )     
        {
            case BUZZ_SINGLE:
                switch( stcRunBuff.enStates )
                {
                    case BUZZ_ON:       //播放
                        if( --stcRunBuff.u8Duration == 0u )
                        {
                            stcRunBuff.u8Buzzfreq = EOT;    //音频设为“休止符”
                            stcRunBuff.enStates = BUZZ_DIS; //状态设置为“静音”
                        }
                        BuzzOn();                       
                        break;
                        
                    default:        //静音或其他干扰状态
                        BuzzInit();
                        break;
                }
                break;
                
            case BUZZ_MUSIC:
                switch(stcRunBuff.enStates)
                {
                    case BUZZ_ON:       //播放                    
                        if( stcRunBuff.u8Duration > 0u )            //曲谱播放时长非0
                        {
                            if( --stcRunBuff.u8Duration == 0u ) //播放时长递减为0
                            {
                                BuzzOnOver();
                            }
//                            INFO_MSG("BUZZ_ON : %d\r\n",stcRunBuff.u8Duration);
                            BuzzOn();
                        }
                        else                        //找到曲谱的播放时长为0
                        {
                            BuzzOnOver();
                        }
                        break;
                        
                    case BUZZ_OFF:      //停顿
                        if( stcRunBuff.u8Duration > 0u )            //曲谱停顿时长非0
                        {
                            if( --stcRunBuff.u8Duration == 0u ) //播放时长递减为0
                            {
                                BuzzOffOver();
                            }
                            INFO_MSG("BUZZ_OFF : %d\r\n",stcRunBuff.u8Duration);
                            BuzzOff();
                        }
                        else
                        {
                            BuzzOffOver();
                        }
                        break;
                        
                    default:        //静音或其他干扰状态
                        BuzzInit();     //初始化蜂鸣器    
                        break;
                }
                break;
            default:
                BuzzInit();     //初始化蜂鸣器    
                break;
        }
#else
        if( BUZZ_ON == stcRunBuff.enStates )    //播放状态
        {
            if( stcRunBuff.u8RemainTime > 0u )  
            {
                stcRunBuff.u8RemainTime--;      //蜂鸣保持时间非零继续递减
            }
            else
            {
                stcRunBuff.u8BuzzData = 0u;     //传输蜂鸣数据为0
                stcRunBuff.enStates = BUZZ_DIS;
            }
        }
        else                                //静音状态
        {
            stcRunBuff.u8BuzzData = 0u;     //传输蜂鸣数据为0
        }
#endif

    }
}

#endif
