/**************************************************************************/
/*                                                                        */
/*       Copyright (c) Microsoft Corporation. All rights reserved.        */
/*                                                                        */
/*       This software is licensed under the Microsoft Software License   */
/*       Terms for Microsoft Azure RTOS. Full text of the license can be  */
/*       found in the LICENSE file at https://aka.ms/AzureRTOS_EULA       */
/*       and in the root directory of this software.                      */
/*                                                                        */
/**************************************************************************/

#include "lx_nand_hds007_driver.h"

#if 1

static UINT  hds007_nand_driver_read(ULONG block, ULONG page, ULONG *destination, ULONG words);
static UINT  hds007_nand_driver_write(ULONG block, ULONG page, ULONG *source, ULONG words);

static UINT  hds007_nand_driver_block_erase(ULONG block, ULONG erase_count);
static UINT  hds007_nand_driver_block_erased_verify(ULONG block);
static UINT  hds007_nand_driver_page_erased_verify(ULONG block, ULONG page);

static UINT  hds007_nand_driver_block_status_get(ULONG block, UCHAR *bad_block_byte);
static UINT  hds007_nand_driver_block_status_set(ULONG block, UCHAR bad_block_byte);

static UINT  hds007_nand_driver_extra_bytes_get(ULONG block, ULONG page, UCHAR *destination, UINT size);
static UINT  hds007_nand_driver_extra_bytes_set(ULONG block, ULONG page, UCHAR *source, UINT size);

static UINT  hds007_nand_driver_system_error(UINT error_code, ULONG block, ULONG page);

static UINT  hds007_nand_flash_driver_pages_read(ULONG block, ULONG page, UCHAR *main_buffer, UCHAR *spare_buffer, ULONG pages);
static UINT  hds007_nand_flash_driver_pages_write(ULONG block, ULONG page, UCHAR *main_buffer, UCHAR *spare_buffer, ULONG pages);
static UINT  hds007_nand_flash_driver_pages_copy(ULONG source_block, ULONG source_page, ULONG destination_block, ULONG destination_page, ULONG pages, UCHAR *data_buffer);

/* USER CODE BEGIN 0 */
#include "w25n01gv.h"

#define BAD_BLOCK_POSITION                  0           /*      0 is the bad block byte postion                         */
#define EXTRA_BYTE_POSITION                 2           /*      0 is the extra bytes starting byte postion              */
#define ECC_BYTE_POSITION                   8           /*      8 is the ECC starting byte position						*/

//#define NAND_LOG(fmt, args...)	Debug_Printf(0,LOG_DEBUG_LEVEL,fmt, ##args)
#define NAND_LOG(fmt, args...)

UINT8 nandBuf[2112];

extern uint8_t Spi_Transmit(FlashCmdFormat _cmd,uint32_t _wlen,uint8_t *_wbuf,uint32_t _rlen,uint8_t *_rbuf);
/* USER CODE END 0 */

#ifndef WORDS_PER_PHYSICAL_PAGE
#define WORDS_PER_PHYSICAL_PAGE 512
#endif

static UCHAR  nandFlashBuffer[WORDS_PER_PHYSICAL_PAGE];

UINT lx_stm32_nand_hds007_driver_initialize(LX_NAND_FLASH *nand_flash)
{
  UINT ret = LX_SUCCESS;

  /* USER CODE BEGIN Init_Section_0 */

  /*USER CODE END Init_Section_0 */

  nand_flash->lx_nand_flash_total_blocks =                  TOTAL_BLOCKS;
  nand_flash->lx_nand_flash_pages_per_block =               PHYSICAL_PAGES_PER_BLOCK;
  nand_flash->lx_nand_flash_bytes_per_page =                BYTES_PER_PHYSICAL_PAGE;

  nand_flash -> lx_nand_flash_spare_data1_offset =          SPARE_DATA1_OFFSET;
  nand_flash -> lx_nand_flash_spare_data1_length =          SPARE_DATA1_LENGTH;

  nand_flash -> lx_nand_flash_spare_data2_offset =          SPARE_DATA2_OFFSET;
  nand_flash -> lx_nand_flash_spare_data2_length =          SPARE_DATA2_LENGTH;

  nand_flash -> lx_nand_flash_spare_total_length =          SPARE_BYTES_PER_PAGE;

  /* USER CODE BEGIN Init_Section_1 */

  /*USER CODE END Init_Section_1 */

  nand_flash->lx_nand_flash_driver_read =                   hds007_nand_driver_read;
  nand_flash->lx_nand_flash_driver_write =                  hds007_nand_driver_write;

  nand_flash->lx_nand_flash_driver_block_erase =            hds007_nand_driver_block_erase;
  nand_flash->lx_nand_flash_driver_block_erased_verify =    hds007_nand_driver_block_erased_verify;
  nand_flash->lx_nand_flash_driver_page_erased_verify =     hds007_nand_driver_page_erased_verify;

  nand_flash->lx_nand_flash_driver_block_status_get =       hds007_nand_driver_block_status_get;
  nand_flash->lx_nand_flash_driver_block_status_set =       hds007_nand_driver_block_status_set;

  nand_flash->lx_nand_flash_driver_extra_bytes_get =        hds007_nand_driver_extra_bytes_get;
  nand_flash->lx_nand_flash_driver_extra_bytes_set =        hds007_nand_driver_extra_bytes_set;

  nand_flash->lx_nand_flash_driver_system_error =           hds007_nand_driver_system_error;

  nand_flash->lx_nand_flash_driver_pages_read =             hds007_nand_flash_driver_pages_read;
  nand_flash->lx_nand_flash_driver_pages_write =            hds007_nand_flash_driver_pages_write;
  nand_flash->lx_nand_flash_driver_pages_copy =             hds007_nand_flash_driver_pages_copy;

  /* USER CODE BEGIN Init_Section_2 */

  /*USER CODE END Init_Section_2 */

  nand_flash->lx_nand_flash_page_buffer =  &nandFlashBuffer[0];

  /* USER CODE BEGIN Init_Section_3 */

  /*USER CODE END Init_Section_3 */
  return ret;

}

static UINT  hds007_nand_driver_read(ULONG block, ULONG page, ULONG *destination, ULONG words)
{

  UINT ret = LX_SUCCESS;

 /* USER CODE BEGIN driver_read */
	UINT    addr;

	if(block < 512)
		block += 512;

	NAND_LOG("read block:%d --- page:%d --- words:%d\r\n", block,page,words);
	addr = block << 6;
	addr += page;
	addr <<= 12;
	W25N01GV_ReadPage(addr,2048,(UINT8 *)destination);
 /* USER CODE END driver_read */

  return ret;
}

static UINT  hds007_nand_driver_write(ULONG block, ULONG page, ULONG *source, ULONG words)
{
  UINT ret = LX_SUCCESS;

 /* USER CODE BEGIN driver_write */
	UCHAR   *flash_spare_address;
	UINT    bytes_computed;
	UINT    ecc_bytes =0;
	UCHAR   new_ecc_buffer[24];
	UCHAR   *new_ecc_buffer_ptr = new_ecc_buffer;
	UCHAR   *ecc_buffer_ptr = new_ecc_buffer_ptr;
	ULONG   *page_ptr = (ULONG   *)nandBuf;

	UINT    addr;

	if(block < 512)
		block += 512;
	NAND_LOG("write block:%d --- page:%d --- words:%d\r\n", block,page,words);
	addr = block << 6;
	addr += page;
	addr <<= 12;

	/* Loop to write flash.  */
	while (words--)
	{
		/* Copy word.  */
		*page_ptr++ =  *source++;
	}

	/* Loop to compute the ECC over the entire NAND flash page.  */
	bytes_computed =  0;

	while (bytes_computed < BYTES_PER_PHYSICAL_PAGE)
	{

	/* Compute the ECC for this 256 byte piece of the page.  */
	_lx_nand_flash_256byte_ecc_compute((UCHAR *)page_ptr, (UCHAR *)new_ecc_buffer_ptr);

	/* Move to the next 256 byte portion of the page.  */
	bytes_computed =  bytes_computed + 256;

	/* Move the page buffer forward.  */
	page_ptr =  page_ptr + 64;

	ecc_bytes = ecc_bytes + 3;

	/* Move the ECC buffer forward, note there are 3 bytes of ECC per page. */
	new_ecc_buffer_ptr =   new_ecc_buffer_ptr + 3;
	}

	/* Setup destination pointer in the spare area.  */
	flash_spare_address =  (UCHAR   *)&nandBuf[2048 + ECC_BYTE_POSITION];
	while(ecc_bytes--)
	{

		*flash_spare_address++ =  *ecc_buffer_ptr++;

	}
	W25N01GV_WritePage(addr,2112,nandBuf);
 /* USER CODE END driver_write */

  return ret;
}

static UINT  hds007_nand_driver_block_erase(ULONG block, ULONG erase_count)
{
  UINT ret = LX_SUCCESS;

  if(block < 512)
  		block += 512;
  NAND_LOG("block_erase block:%d --- page:%d\r\n", block,erase_count);
	W25N01GV_Erase(block << 6);


  return ret;
}

static UINT  hds007_nand_driver_block_erased_verify(ULONG block)
{
  UINT ret = LX_SUCCESS;

 /* USER CODE BEGIN block_erase_verify */
	UINT   i;

	if(block < 512)
		block += 512;
	NAND_LOG("block_erased_verify block:%d\r\n", block);

	for(i = 0;i< 64;i++)
	{
		if(hds007_nand_driver_page_erased_verify( block, i) != LX_SUCCESS)
		{
			ERROR_MSG("verify\r\n");
			return(LX_ERROR);
		}

	}
 /* USER CODE END block_erase_verify */

  return ret;
}

static UINT  hds007_nand_driver_page_erased_verify(ULONG block, ULONG page)
{
  UINT ret = LX_SUCCESS;

 /* USER CODE BEGIN page_erased_verify */
	ULONG   *word_ptr;
	ULONG   words;
	UINT    addr;

	if(block < 512)
		block += 512;

	addr = block << 6;
	addr += page;
	addr <<= 12;
	NAND_LOG("page_erased_verify block:%d\r\n", block);

	W25N01GV_ReadPage(addr,WORDS_PER_PHYSICAL_PAGE,nandBuf);
	/* Pickup the pointer to the first word of the block's page.  */
	word_ptr =  (ULONG *) nandBuf;

	/* Calculate the number of words in a block.  */
	words =  WORDS_PER_PHYSICAL_PAGE;

	/* Loop to check if the page is erased.  */
	while (words--)
	{

		/* Is this word erased?  */
		if (*word_ptr++ != 0xFFFFFFFF)
		return(LX_ERROR);
	}
 /* USER CODE END page_erased_verify */

  return ret;
}

static UINT  hds007_nand_driver_block_status_get(ULONG block, UCHAR *bad_block_byte)
{
  UINT ret = LX_SUCCESS;

 /* USER CODE BEGIN block_status_get */
	UINT32 addr;

	//INFO_MSG("block_status_get block:%d\r\n", block);
	if(block < 512)
		block += 512;
	addr = block << 6;
	addr <<= 12;
	addr += 2048;
	W25N01GV_ReadPage(addr,4,nandBuf);

	/* Pickup the bad block byte and return it.  */
	*bad_block_byte =  nandBuf[ 0];
	//INFO_MSG("block bad byte:%d\r\n", *bad_block_byte);
 /* USER CODE END block_status_get*/

  return ret;
}

static UINT  hds007_nand_driver_block_status_set(ULONG block, UCHAR bad_block_byte)
{
  UINT ret = LX_SUCCESS;

 /* USER CODE BEGIN block_status_set */
	UINT32 addr;
	if(block < 512)
		block += 512;
	NAND_LOG("block_status_set block:%ld\r\n", block);

	addr = block << 6;
	addr <<= 12;
	addr += 2048;
	W25N01GV_ReadPage(addr,64,nandBuf);
	/* Set the bad block byte.  */
	nandBuf[0] =  bad_block_byte;
	W25N01GV_WritePage(addr,64,nandBuf);
 /* USER CODE END block_status_set */

  return ret;
}

static UINT  hds007_nand_driver_extra_bytes_get(ULONG block, ULONG page, UCHAR *destination, UINT size)
{
  UINT ret = LX_SUCCESS;

 /* USER CODE BEGIN extra_bytes_get */
	UCHAR   *source;

	UINT32 addr;
	if(block < 512)
		block += 512;
	NAND_LOG("extra_bytes_get block:%d,size:%d\r\n", block,size);

	addr = block << 6;
	addr += page;
	addr <<= 12;
	addr += 2048;
	W25N01GV_ReadPage(addr,64,nandBuf);
	/* Setup source pointer in the spare area.  */
	source =  (UCHAR *) &nandBuf[0];

	/* Loop to return the extra bytes requested.  */
	while (size--)
	{

	  /* Retrieve an extra byte from the spare area.  */
	  *destination++ =  *source++;
	}
 /* USER CODE END extra_bytes_get */

  return ret;
}

static UINT  hds007_nand_driver_extra_bytes_set(ULONG block, ULONG page, UCHAR *source, UINT size)
{
  UINT ret = LX_SUCCESS;

 /* USER CODE BEGIN extra_bytes_set */
	UCHAR   *destination;

	UINT32 addr;
	if(block < 512)
		block += 512;
	NAND_LOG("extra_bytes_set block:%d,size:%d---%x-%x\r\n", block,size,source[0],source[1]);

	addr = block << 6;
	addr += page;
	addr <<= 12;
	addr += 2048;
	W25N01GV_ReadPage(addr,64,nandBuf);

	destination =  (UCHAR *) nandBuf;

	/* Loop to set the extra bytes.  */
	while (size--)
	{

		/* Set an extra byte in the spare area.  */
		*destination++ =  *source++;
	}
	W25N01GV_WritePage(addr,64,nandBuf);
 /* USER CODE END extra_bytes_set */

  return ret;
}

static UINT  hds007_nand_driver_system_error(UINT error_code, ULONG block, ULONG page)
{
  UINT ret = LX_SUCCESS;

 /* USER CODE BEGIN system_error */
  ERROR_MSG("error_code---0x%X\r\n",error_code);
 /* USER CODE END system_error */

  return ret;
}

static UINT  hds007_nand_flash_driver_pages_read(ULONG block, ULONG page, UCHAR *main_buffer, UCHAR *spare_buffer, ULONG pages)
{
  UINT ret = LX_SUCCESS;

 /* USER CODE BEGIN driver_pages_read */
	UINT    i;
	UINT32 pageAddr,status;
	FlashCmdFormat _cmd;

	if(block < 512)
		block += 512;
	//INFO_MSG("read pages block:%d --- page:%d\r\n", block,page);
	for (i = 0; i < pages; i++)
	{
		pageAddr = (block << 6) + page + i;

		_cmd.instruct = 0x13;
		_cmd.addr = 0;
		_cmd.addrSize = 3;
		_cmd.addr |= (pageAddr &0XFFFF);

		Spi_Transmit(_cmd,0,NULL,0,NULL);
		//HAL_Delay(100);
		status = W25N01GV_WaitBusy();

		//	INFO_MSG("Read status = 0x%X\r\n", status);

		_cmd.instruct = 0x03;
		_cmd.addrSize = 3;
		_cmd.addr = 0;

		Spi_Transmit(_cmd,0,NULL,2048,(main_buffer + i * BYTES_PER_PHYSICAL_PAGE));

		_cmd.instruct = 0x03;
		_cmd.addrSize = 3;
		_cmd.addr = ((2048) << 8);

		Spi_Transmit(_cmd,0,NULL,SPARE_BYTES_PER_PAGE,(spare_buffer + i * SPARE_BYTES_PER_PAGE));
	}
 /* USER CODE END driver_pages_read */

  return ret;
}

static UINT  hds007_nand_flash_driver_pages_write(ULONG block, ULONG page, UCHAR *main_buffer, UCHAR *spare_buffer, ULONG pages)
{
  UINT ret = LX_SUCCESS;

 /* USER CODE BEGIN driver_pages_write */
	UINT    i;
	UINT    status = LX_SUCCESS;
	UINT32 pageAddr;
	FlashCmdFormat _cmd;

	if(block < 512)
		block += 512;

	NAND_LOG("Write block = %d page = %d\r\n", block,page);
	for (i = 0; i < pages; i++)
	{

		pageAddr = (block << 6) + page + i;
		W25N01GV_WriteEnable(1);

		_cmd.instruct = 0x84;
		_cmd.addrSize = 2;
		_cmd.addr = 0;
		Spi_Transmit(_cmd,2048,(main_buffer + i * BYTES_PER_PHYSICAL_PAGE),0,NULL);

		status = W25N01GV_WaitBusy();
		_cmd.instruct = 0x84;
		_cmd.addrSize = 2;
		_cmd.addr = 2048;
		Spi_Transmit(_cmd,64,spare_buffer + i * SPARE_BYTES_PER_PAGE,0,NULL);
		//HAL_Delay(100);
		status = W25N01GV_WaitBusy();

		_cmd.instruct = 0x10;
		_cmd.addr = 0;
		_cmd.addrSize = 3;
		_cmd.addr |= (pageAddr &0XFFFF);
		Spi_Transmit(_cmd,0,NULL,0,NULL);

		status = W25N01GV_WaitBusy();
//		INFO_MSG("Write status = 0x%X\r\n", status);
	}
 /* USER CODE END driver_pages_write */

  return ret;
}

static UINT  hds007_nand_flash_driver_pages_copy(ULONG source_block, ULONG source_page, ULONG destination_block, ULONG destination_page, ULONG pages, UCHAR *data_buffer)
{
  UINT ret = LX_SUCCESS;

  UINT    i;

  if(source_block < 512)
	  source_block += 512;
  if(destination_block < 512)
	  destination_block += 512;


  for (i = 0; i < pages; i++)
  {
	  ret = hds007_nand_flash_driver_pages_read(source_block, source_page + i, data_buffer, data_buffer + BYTES_PER_PHYSICAL_PAGE, 1);

      if (ret != LX_SUCCESS && ret != LX_NAND_ERROR_CORRECTED)
      {
          break;
      }
      ret = hds007_nand_flash_driver_pages_write(destination_block, destination_page + i, data_buffer, data_buffer + BYTES_PER_PHYSICAL_PAGE, 1);

      if (ret != LX_SUCCESS)
      {
          break;
      }

  }


  return ret;
}

LX_NAND_FLASH       nand_flash;
///* Memory buffer size should be at least 7 * total block count + 2 * page size,
//   that is 7 * 1024 + 2 * 528 = 8224 bytes *///2112*2= 4224--11392
ULONG               lx_buffer[(11392 * 2) / sizeof (ULONG)];
//ULONG               *lx_memory_buffer = (ULONG  *)XSPI1_BASE;
UINT8 fBuf[2112];
UINT32 NandFlash_Open()
{
    UINT32 status;
    status =  lx_nand_flash_open(&nand_flash, "hds007 nand flash", lx_stm32_nand_hds007_driver_initialize, lx_buffer, sizeof(lx_buffer));
    if (status != LX_SUCCESS)
	{
		ERROR_MSG("open nand flash fail 0x%X\r\n",status);
		status =  _lx_nand_flash_format(&nand_flash, "hds007 nand flash", lx_stm32_nand_hds007_driver_initialize, lx_buffer, sizeof(lx_buffer));
		if (status != LX_SUCCESS)
		{

			ERROR_MSG("forma nand flash fail\r\n");

			return status;
		}

	}
    return status;
}
UINT32 NandFlash_close()
{
    UINT32 status = 0;
    lx_nand_flash_close(&nand_flash);
    return status;
}
UINT32 NandFlash_Read(UINT32 _addr,UINT32 _len,UINT8* _buff)
{
    UINT32 i,status,cnt,pages;
    cnt= 0;
    while(_len > 0)
    {
        if(_len < 2048)
        {
            status = lx_nand_flash_sector_read(&nand_flash, _addr, (VOID *)fBuf);
            if(status != LX_SUCCESS)
            {
                ERROR_MSG("Read status = %x\r\n",status);
                return status;
            }
            for (i = 0; i < _len; i++)
            {
                _buff[cnt + i] = fBuf[i];
            }
            return 0;
        }
        else
        {
            pages = _len/2048;
            INFO_MSG("Statr Read pages = %d\r\n",pages);
            lx_nand_flash_sectors_read(&nand_flash, _addr, (VOID *)_buff, pages);
            cnt = pages * 2048;
            _addr += pages;
            _len -= cnt;
            INFO_MSG("Statr Read _len = %d\r\n",_len);
        }
    }

	return 0;
}
UINT32 NandFlash_Write(UINT32 _addr,UINT32 _len,UINT8* _buff)
{
    UINT32 i,status,cnt,pages;
    cnt= 0;
    while(_len > 0)
    {
        if(_len < 2048)
        {
            for (i = 0; i < _len; i++)
            {
                fBuf[i] = _buff[cnt + i];
            }
            status = lx_nand_flash_sector_write(&nand_flash, _addr, (VOID *)fBuf);
            if(status != LX_SUCCESS)
            {
                ERROR_MSG("Read status = %x\r\n",status);
                return status;
            }

            return 0;
        }
        else
        {
            pages = _len/2048;
            lx_nand_flash_sectors_write(&nand_flash, _addr, (VOID *)_buff, pages);
            cnt = pages * 2048;
            _addr += pages;
            _len -= cnt;
        }
    }
	return 0;
}
UINT32 NandFlash_WriteSectors(UINT32 _page,UINT32 _len,UINT8* _buff)
{

    lx_nand_flash_sectors_write(&nand_flash, _page, (VOID *)_buff, _len);

	return 0;
}
UINT32 NandFlash_ReadSectors(UINT32 _page,UINT32 _len,UINT8* _buff)
{
    lx_nand_flash_sectors_read(&nand_flash, _page, (VOID *)_buff, _len);

	return 0;
}

#endif

