#ifndef __TUYAWIFI_H
#define __TUYAWIFI_H

#ifdef __cplusplus
 extern "C" {
#endif
	 
#include "Debug.h"
#include "main.h"
#include "System.h"

 //=============================================================================
 //dp数据点类型
 //=============================================================================
 #define         DP_TYPE_RAW                     0x00        //RAW 类型
 #define         DP_TYPE_BOOL                    0x01        //bool 类型
 #define         DP_TYPE_VALUE                   0x02        //value 类型
 #define         DP_TYPE_STRING                  0x03        //string 类型
 #define         DP_TYPE_ENUM                    0x04        //enum 类型
 #define         DP_TYPE_BITMAP                  0x05        //fault 类型

 //=============================================================================
 //WIFI工作状态
 //=============================================================================
 #define         SMART_CONFIG_STATE              0x00
 #define         AP_STATE                        0x01
 #define         WIFI_NOT_CONNECTED              0x02
 #define         WIFI_CONNECTED                  0x03
 #define         WIFI_CONN_CLOUD                 0x04
 #define         WIFI_LOW_POWER                  0x05
 #define         SMART_AND_AP_STATE              0x06
 #define         WIFI_SATE_UNKNOW                0xff
 //=============================================================================
 //wifi配网的方式
 //=============================================================================
 #define         SMART_CONFIG                    0x0
 #define         AP_CONFIG                       0x1

 //=============================================================================
 //wifi复位状态
 //=============================================================================
 #define         RESET_WIFI_ERROR                0
 #define         RESET_WIFI_SUCCESS              1

 //=============================================================================
 //wifi配置复位状态
 //=============================================================================
 #define         SET_WIFICONFIG_ERROR            0
 #define         SET_WIFICONFIG_SUCCESS          1

 //=============================================================================
 //MCU固件升级状态
 //=============================================================================
 #define         FIRM_STATE_UN_SUPPORT           0x00                            //不支持 MCU 升级
 #define         FIRM_STATE_WIFI_UN_READY        0x01                            //模块未就绪
 #define         FIRM_STATE_GET_ERROR            0x02                            //云端升级信息查询失败
 #define         FIRM_STATE_NO                   0x03                            //无需升级（云端无更新版本）
 #define         FIRM_STATE_START                0x04                            //需升级，等待模块发起升级操作

 //=============================================================================
 //WIFI和mcu的工作方式
 //=============================================================================
 #define         UNION_WORK                      0x0                             //mcu模块与wifi配合处理
 #define         WIFI_ALONE                      0x1                             //wifi模块自处理

 //=============================================================================
 //系统工作模式
 //=============================================================================
 #define         NORMAL_MODE                     0x00                            //正常工作状态
 #define         FACTORY_MODE                    0x01                            //工厂模式
 #define         UPDATE_MODE                     0x02                            //升级模式

 //=============================================================================
 //配网方式选择
 //=============================================================================
 #define         CONFIG_MODE_DEFAULT             "0"                             //默认配网方式
 #define         CONFIG_MODE_LOWPOWER            "1"                             //低功耗配网方式
 #define         CONFIG_MODE_SPECIAL             "2"                             //特殊配网方式


 //=============================================================================
 //Data frame type
 //=============================================================================
 #define         HEAT_BEAT_CMD                   0                               //心跳包
 #define         PRODUCT_INFO_CMD                1                               //产品信息
 #define         WORK_MODE_CMD                   2                               //查询MCU 设定的模块工作模式
 #define         WIFI_STATE_CMD                  3                               //wifi工作状态
 #define         WIFI_RESET_CMD                  4                               //重置wifi
 #define         WIFI_MODE_CMD                   5                               //选择smartconfig/AP模式
 #define         DATA_QUERT_CMD                  6                               //命令下发
 #define         STATE_UPLOAD_CMD                7                               //状态上报
 #define         STATE_QUERY_CMD                 8                               //状态查询
 #define         UPDATE_START_CMD                0x0a                            //升级开始
 #define         UPDATE_TRANS_CMD                0x0b                            //升级传输
 #define         GET_ONLINE_TIME_CMD             0x0c                            //获取系统时间(格林威治时间)
 #define         FACTORY_MODE_CMD                0x0d                            //进入产测模式
 #define         WIFI_TEST_CMD                   0x0e                            //wifi功能测试
 #define         GET_LOCAL_TIME_CMD              0x1c                            //获取本地时间
 #define         WEATHER_OPEN_CMD                0x20                            //打开天气
 #define         WEATHER_DATA_CMD                0x21                            //天气数据
 #define         STATE_UPLOAD_SYN_CMD            0x22                            //状态上报（同步）
 #define         STATE_UPLOAD_SYN_RECV_CMD       0x23                            //状态上报结果通知（同步）
 #define         HEAT_BEAT_STOP                  0x25                            //关闭WIFI模组心跳
 #define         STREAM_TRANS_CMD                0x28                            //流数据传输
 #define         GET_WIFI_STATUS_CMD             0x2b                            //获取当前wifi联网状态
 #define         WIFI_CONNECT_TEST_CMD           0x2c                            //wifi功能测试(连接指定路由)
 #define         GET_MAC_CMD                     0x2d                            //获取模块mac
 #define         GET_IR_STATUS_CMD               0x2e                            //红外状态通知
 #define         IR_TX_RX_TEST_CMD               0x2f                            //红外进入收发产测
 #define         MAPS_STREAM_TRANS_CMD           0x30                            //流数据传输(支持多张地图)
 #define         FILE_DOWNLOAD_START_CMD         0x31                            //文件下载启动
 #define         FILE_DOWNLOAD_TRANS_CMD         0x32                            //文件下载数据传输
 #define         MODULE_EXTEND_FUN_CMD           0x34                            //模块拓展服务
 #define         BLE_TEST_CMD                    0x35                            //蓝牙功能性测试（扫描指定蓝牙信标）
 #define         GET_VOICE_STATE_CMD             0x60                            //获取语音状态码
 #define         MIC_SILENCE_CMD                 0x61                            //MIC静音设置
 #define         SET_SPEAKER_VOLUME_CMD          0x62                            //speaker音量设置
 #define         VOICE_TEST_CMD                  0x63                            //语音模组音频产测
 #define         VOICE_AWAKEN_TEST_CMD           0x64                            //语音模组唤醒产测
 #define         VOICE_EXTEND_FUN_CMD            0x65                            //语音模组扩展功能



 /******************************************************************************
                         1:dp数据点序列号重新定义
           **此为自动生成代码,如在开发平台有相关修改请重新下载MCU_SDK**
 ******************************************************************************/
 //MCU信息(可下发可上报)
 //备注:
 #define DPID_MCU_INFO 101
 //设备状态上报(可下发可上报)
 //备注:
 #define DPID_DEVICE_STATUS 102
 //LCD背光(可下发可上报)
 //备注:调整显示背光
 #define DPID_BACKLIGHT 103
 //蜂鸣器开关(可下发可上报)
 //备注:
 #define DPID_BUZZER 104

 //=============================================================================
 //下发命令
 //=============================================================================
 typedef struct {
   unsigned char dp_id;                              //dp序号
   unsigned char dp_type;                            //dp类型
 } DOWNLOAD_CMD_S;

void TuyaWifi_Init(void);
UINT8 TuyaWifi_HeartBeatResponse(UINT8 _flag);
UINT8 TuyaWifi_ProductInfResponse();
UINT8 TuyaWifi_WorkModeResponse();
UINT8 TuyaWifi_WorkStateResponse();

UINT8 TuyaWifi_DpBoolUpdate(UINT8 _dpid,UINT8 _value);
UINT8 TuyaWifi_DpValueUpdate(UINT8 _dpid,UINT32 _value);
UINT8 TuyaWifi_DpStringUpdate(UINT8 _dpid,const UINT8 _value[],UINT16 _len);
UINT8 TuyaWifi_DpHandle(UINT8 value[]);
UINT8 TuyaWifi_ResetNetwork(UINT8 _value);


#ifdef __cplusplus
}
#endif
#endif
















